@@ -29,6 +29,8 @@
 extern "C" {
 #endif
 
+#include <spa/support/plugin.h>
+#include <spa/support/loop.h>
 #include <spa/utils/hook.h>
 
 #include "config.h"
@@ -45,6 +47,31 @@ extern "C" {
 #define MIN_LATENCY	128
 #define MAX_LATENCY	1024
 
+#define OBJECT_MANAGER_INTROSPECT_XML                                          \
+	DBUS_INTROSPECT_1_0_XML_DOCTYPE_DECL_NODE                                  \
+	"<node>\n"                                                                 \
+	" <interface name=\"org.freedesktop.DBus.ObjectManager\">\n"               \
+	"  <method name=\"GetManagedObjects\">\n"                                  \
+	"   <arg name=\"objects\" direction=\"out\" type=\"a{oa{sa{sv}}}\"/>\n"    \
+	"  </method>\n"                                                            \
+	"  <signal name=\"InterfacesAdded\">\n"                                    \
+	"   <arg name=\"object\" type=\"o\"/>\n"                                   \
+	"   <arg name=\"interfaces\" type=\"a{sa{sv}}\"/>\n"                       \
+	"  </signal>\n"                                                            \
+	"  <signal name=\"InterfacesRemoved\">\n"                                  \
+	"   <arg name=\"object\" type=\"o\"/>\n"                                   \
+	"   <arg name=\"interfaces\" type=\"as\"/>\n"                              \
+	"  </signal>\n"                                                            \
+	" </interface>\n"                                                          \
+	" <interface name=\"org.freedesktop.DBus.Introspectable\">\n"              \
+	"  <method name=\"Introspect\">\n"                                         \
+	"   <arg name=\"data\" direction=\"out\" type=\"s\"/>\n"                   \
+	"  </method>\n"                                                            \
+	" </interface>\n"                                                          \
+	" <node name=\"A2DPSink\"/>\n"                                             \
+	" <node name=\"A2DPSource\"/>\n"                                           \
+	"</node>\n"
+
 #define ENDPOINT_INTROSPECT_XML                                             \
 	DBUS_INTROSPECT_1_0_XML_DOCTYPE_DECL_NODE                           \
 	"<node>"                                                            \
@@ -110,6 +137,10 @@ extern "C" {
 #define HFP_AUDIO_CODEC_CVSD    0x01
 #define HFP_AUDIO_CODEC_MSBC    0x02
 
+#define A2DP_OBJECT_MANAGER_PATH "/MediaEndpoint"
+#define A2DP_SINK_ENDPOINT	A2DP_OBJECT_MANAGER_PATH "/A2DPSink"
+#define A2DP_SOURCE_ENDPOINT	A2DP_OBJECT_MANAGER_PATH "/A2DPSource"
+
 enum spa_bt_profile {
         SPA_BT_PROFILE_NULL =		0,
         SPA_BT_PROFILE_A2DP_SINK =	(1 << 0),
@@ -176,6 +207,8 @@ struct spa_bt_adapter {
 	uint32_t bluetooth_class;
 	uint32_t profiles;
 	int powered;
+	unsigned int endpoints_registered:1;
+	unsigned int application_registered:1;
 };
 
 struct spa_bt_device {
@@ -241,6 +274,7 @@ struct spa_bt_transport {
 	struct spa_list device_link;
 	enum spa_bt_profile profile;
 	enum spa_bt_transport_state state;
+	const struct a2dp_codec *a2dp_codec;
 	int codec;
 	void *configuration;
 	int configuration_len;
