@@ -143,6 +143,9 @@ struct impl {
 
 	int state_dir_fd;
 	char state_dir[PATH_MAX];
+
+	unsigned int scanning:1;
+	unsigned int rescan_pending:1;
 };
 
 struct endpoint_link {
@@ -624,6 +627,8 @@ static void node_destroy(void *object)
 		pw_node_info_free(node->info);
 		node->info = NULL;
 	}
+	free(node->target_node);
+	node->target_node = NULL;
 }
 
 static const struct object_info node_info = {
@@ -1297,6 +1302,11 @@ int sm_media_session_for_each_object(struct sm_media_session *sess,
 int sm_media_session_schedule_rescan(struct sm_media_session *sess)
 {
 	struct impl *impl = SPA_CONTAINER_OF(sess, struct impl, this);
+
+	if (impl->scanning) {
+		impl->rescan_pending = true;
+		return impl->rescan_seq;
+	}
 	if (impl->policy_core)
 		impl->rescan_seq = pw_core_sync(impl->policy_core, 0, impl->last_seq);
 	return impl->rescan_seq;
@@ -1343,6 +1353,8 @@ int sm_media_session_roundtrip(struct sm_media_session *sess)
 	pw_loop_enter(loop);
 	while (!done) {
 		if ((res = pw_loop_iterate(loop, -1)) < 0) {
+			if (res == -EINTR)
+				continue;
 			pw_log_warn(NAME" %p: iterate error %d (%s)",
 				loop, res, spa_strerror(res));
 			break;
@@ -1932,8 +1944,16 @@ static void core_done(void *data, uint32_t id, int seq)
 	if (impl->rescan_seq == seq) {
 		struct sm_object *obj, *to;
 
-		pw_log_trace(NAME" %p: rescan %u %d", impl, id, seq);
-		sm_media_session_emit_rescan(impl, seq);
+		if (!impl->scanning) {
+			pw_log_trace(NAME" %p: rescan %u %d", impl, id, seq);
+			impl->scanning = true;
+			sm_media_session_emit_rescan(impl, seq);
+			impl->scanning = false;
+			if (impl->rescan_pending) {
+				impl->rescan_pending = false;
+				sm_media_session_schedule_rescan(&impl->this);
+			}
+		}
 
 		spa_list_for_each_safe(obj, to, &impl->global_list, link) {
 			pw_log_trace(NAME" %p: obj %p %08x", impl, obj, obj->changed);
@@ -2022,6 +2042,15 @@ static void session_shutdown(struct impl *impl)
 		pw_core_info_free(impl->this.info);
 }
 
+static int sm_pulse_bridge_start(struct sm_media_session *sess)
+{
+	if (pw_context_load_module(sess->context,
+			"libpipewire-module-protocol-pulse",
+			NULL, NULL) == NULL)
+		return -errno;
+	return 0;
+}
+
 static void do_quit(void *data, int signal_number)
 {
 	struct impl *impl = data;
@@ -2038,10 +2067,10 @@ static void do_quit(void *data, int signal_number)
 				"alsa-acp,"		\
 				"alsa-seq,"		\
 				"v4l2,"			\
-				"bluez5,"		\
 				"suspend-node,"		\
 				"policy-node"
-#define DEFAULT_DISABLED	""
+#define EXTRA_ENABLED		""
+#define EXTRA_DISABLED		""
 
 static const struct {
 	const char *name;
@@ -2065,6 +2094,7 @@ static const struct {
 	{ "bluez5", "bluetooth support", sm_bluez5_monitor_start, NULL },
 	{ "suspend-node", "suspend inactive nodes", sm_suspend_node_start, NULL },
 	{ "policy-node", "configure and link nodes", sm_policy_node_start, NULL },
+	{ "pulse-bridge", "accept pulseaudio clients", sm_pulse_bridge_start, NULL },
 };
 
 static int opt_contains(const char *opt, const char *val)
@@ -2078,24 +2108,29 @@ static int opt_contains(const char *opt, const char *val)
 	return 0;
 }
 
-static void show_help(const char *name)
+static bool is_opt_enabled(const char *enabled, const char *disabled, const char *val)
+{
+	return (opt_contains(DEFAULT_ENABLED, val) || opt_contains(enabled, val)) &&
+			!opt_contains(disabled, val);
+}
+
+static void show_help(const char *name, const char *enabled, const char *disabled)
 {
 	size_t i;
 
         fprintf(stdout, "%s [options]\n"
              "  -h, --help                            Show this help\n"
              "      --version                         Show version\n"
-             "  -e, --enabled                         Enabled options (default '%s')\n"
-             "  -d, --disabled                        Disabled options (default '%s')\n"
+             "  -e, --enabled                         Extra enabled options ('%s')\n"
+             "  -d, --disabled                        Extra disabled options ('%s')\n"
              "  -p, --properties                      Extra properties as 'key=value { key=value }'\n",
-	     name, DEFAULT_ENABLED, DEFAULT_DISABLED);
+	     name, enabled, disabled);
 
         fprintf(stdout,
              "\noptions: (*=enabled)\n");
 	for (i = 0; i < SPA_N_ELEMENTS(modules); i++) {
 		fprintf(stdout, "\t  %c %-15.15s: %s\n",
-				opt_contains(DEFAULT_ENABLED, modules[i].name) &&
-				!opt_contains(DEFAULT_DISABLED, modules[i].name) ? '*' : ' ',
+				is_opt_enabled(enabled, disabled, modules[i].name) ? '*' : ' ',
 				modules[i].name, modules[i].desc);
 	}
 }
@@ -2106,8 +2141,8 @@ int main(int argc, char *argv[])
 	const struct spa_support *support;
 	uint32_t n_support;
 	int res = 0, c;
-	const char *opt_enabled = DEFAULT_ENABLED;
-	const char *opt_disabled = DEFAULT_DISABLED;
+	const char *opt_enabled = EXTRA_ENABLED;
+	const char *opt_disabled = EXTRA_DISABLED;
 	const char *opt_properties = NULL;
 	static const struct option long_options[] = {
 		{ "help",	no_argument,		NULL, 'h' },
@@ -2125,7 +2160,7 @@ int main(int argc, char *argv[])
 	while ((c = getopt_long(argc, argv, "hVe:d:p:", long_options, NULL)) != -1) {
 		switch (c) {
 		case 'h':
-			show_help(argv[0]);
+			show_help(argv[0], opt_enabled, opt_disabled);
 			return 0;
 		case 'V':
 			fprintf(stdout, "%s\n"
@@ -2166,7 +2201,12 @@ int main(int argc, char *argv[])
 	pw_loop_add_signal(impl.this.loop, SIGINT, do_quit, &impl);
 	pw_loop_add_signal(impl.this.loop, SIGTERM, do_quit, &impl);
 
-	impl.this.context = pw_context_new(impl.this.loop, NULL, 0);
+	impl.this.context = pw_context_new(impl.this.loop,
+				pw_properties_new(
+					PW_KEY_CONTEXT_PROFILE_MODULES, "default,rtkit",
+					NULL),
+				0);
+
 	if (impl.this.context == NULL)
 		return -1;
 
@@ -2202,8 +2242,7 @@ int main(int argc, char *argv[])
 
 	for (i = 0; i < SPA_N_ELEMENTS(modules); i++) {
 		const char *name = modules[i].name;
-		if (opt_contains(opt_enabled, name) &&
-		    !opt_contains(opt_disabled, name)) {
+		if (is_opt_enabled(opt_enabled, opt_disabled, name)) {
 			if (modules[i].props) {
 				struct pw_properties *props;
 				props = pw_properties_new_string(modules[i].props);
